`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    12:02:31 02/24/2011 
// Design Name: 
// Module Name:    jtag_access 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
//    JTAG functionality, instantiate shift registers for capture and settings,
//    also contains serial interfaces to off chip devices
//
// Function  Description
// ---------------------------------------
//   0     | No Op 
//   1     | JTAG System Reset (JTAG_SYS_RST), Equivalent to power on reset without reprogramming. Instruction only, (Auto reset)
//   2     | DCFEB status reg (32 bits) shift only for reading back without updating
//   3     | DCFEB status reg (32 bits) capture and shift for updated information.
//   4     | Program Comparator DAC -- same as in old CFEB
//   5     | Set Extra L1a Delay (2 bits) -- not used on DCFEB, exist for compatability with old DMB
//   6     | Read FIFO 1 -- ADC data (16 channels x 12 bits = 192 bits) wide X (6 chips x 8 sample)/event deep
//   7     | Set F5, F8, and F9 in one serial loop (daisy chained) for compatability with old DMB
//   8     | Set Pre Block End (4 bits) (not needed in DCFEB) -- not used on DCFEB, exist for compatability with old DMB
//   9     | Set Comparator Mode and Timing (5 bits) same format as old CFEB
//  10     | Set Buckeye Mask for shifting (6 bits) (default 6'b111111) same format as old CFEB, one bit for each chip
//  11     | Shift data to/from Buckeye chips (48 bits/chip)
//  12     | Set ADC configuration MASK (12 bits)
//  13     | Command to initialize ADC -- Instruction only, (Auto reset)
//  14     | Shift data and write to ADC configuration memory (26 bits)
//  15     | Command to restart pipeline -- Instruction only, (Auto reset)
//  16     | Set pipeline depth (9 bits)
//  17     | Set TTC source (2 bits) Selects 0:FF_EMU_mode, 1:FF_EEM_mode, or 2:Skew_Clr_mode
//  18     | Set calibration mode to external. (only for optical modes) -- Instruction only, (Auto reset)
//  19     | Set calibration mode to internal. (only for optical modes) -- Instruction only, (Auto reset)
//  20     | Set number of samples to readout (7 bits).
//  21     | Write word to BPI interface write FIFO (16 bits).
//  22     | Read word from BPI readback FIFO (16 bits).
//  23     | Read status word from BPI interface (16 bits).
//  24     | Read BPI timer (32 bits).
//  25     | Reset BPI interface. -- Instruction only, (Auto reset)
//  26     | Disable BPI processing. -- Instruction only, persisting
//  27     | Enable BPI processing. -- Instruction only, persisting
//  28     | Comparator Clock phase register (4-bits, 0-15).
//  29     | TMB transmit mode (2-bits, 0: comparator data, 1: fixed patterns, 2: counters, 3: randoms, 4: comparator data, 5: half strip patterns).
//  30     | TMB half strips for injecting patterns into the optical serial data stream for transmit mode 5. (30-bits, 5 per layer) {ly6,...ly1}
//  31     | TMB layer mask to indicat the active layers for half strip patterns in transmit mode 5. (6-bits, 1 per layer)
//  32     | Set DAQ rate to 1 GbE (1.25Gbps line rate) -- Instruction only, (Auto reset)
//  33     | Set DAQ rate to 2.56 GbE (3.2Gbps line rate) -- Instruction only, (Auto reset)
//  34     | Program Calibration DAC -- same style as Comparator DAC
//  35     | Send Control Byte to the MAX 1271 ADC (and conversion clocks)
//  36     | Read back the MAX 1271 ADC conversion stored in the SPI return register.
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module jtag_access(
    input CLK20,             // 20 MHz Clock
    input CLK40,             // 40 MHz Clock
    input CLK120,            // 120 MHz Clock
    input FSTCLK,            // Fast Clock
    input RST,               // Reset default state
    input [6:1] BKY_RTN,     // Serial data returned from amplifiers
    input [15:0] DCFEB_STATUS, // Status word
    input [191:0] ADCDATA,   // Data out of pipeline
    input [15:0] BPI_RBK_FIFO, // Data read back from BPI PROM
    input [15:0] BPI_STATUS, // STATUS word for BPI interface
    input [31:0] BPI_TIMER,  // Timer for BPI commands
	 output JTAG_SYS_RST,      // JTAG initiated system reset 
    output reg RDFIFO,       // Advance fifo to next word
	 output JTAG_RD_MODE,     // JTAG read out mode for FIFO1 
    output [1:0] XL1DLYSET,  // Extra L1A delay setting [1:0]
    output [3:0] LOADPBLK,   // Pre-blockend bits [3:0] not used in DCFEB
    output [1:0] COMP_MODE,  // comparator mode bits [1:0]
    output [2:0] COMP_TIME,  // comparator timing bits [2:0]
	 output CDAC_ENB,         // Comparator DAC enable
	 output CALDAC_ENB,       // Calibration DAC enable
	 output CALADC_ENB,       // Calibration ADC enable
	 output SPI_CK,           // SPI clock
	 output SPI_DAT,          // SPI data
	 input  SPI_RTN,          // Return data from SPI devices
	 output reg CAL_MODE,     // Calibration mode (0: external pulsing, 1: internal pulsing)
    output [6:1] TO_BKY,     // Serial data sent to amplifiers
    output [6:1] BKY_CLK,    // Shift clock for amplifiers
    output reg ADC_WE,       // Write enable for ADC configuration memory
    output [25:0] ADC_MEM,   // Data word for ADC configuration memory
    output [11:0] ADC_MASK,  // Mask for ADCs to configure
    output ADC_INIT,         // ADC initialization signal
    output JC_ADC_CNFG,      // JTAG Controll of ADC configuration memory
	 output RSTRT_PIPE,       // Restart pipeline on JTAG command
    output [8:0] PDEPTH,     // Pipeline Depth register (9 bits)
    output [1:0] TTC_SRC,    // TTC source register (2 bits)
    output [15:0] BPI_WRT_FIFO,   // Data word for ADC configuration memory
    output reg BPI_WE,       // Write enable for BPI Write FIFO
    output reg BPI_RDENA,    // Read enable for BPI Readback FIFO
    output BPI_RESET,        // Reset signal for BPI interface (does not reset the PROM)
    output BPI_DISABLE,      // Disable BPI processing
    output BPI_ENABLE,       // Enable BPI processing
	 output [6:0] SAMP_MAX,   // Number of samples to readout minus 1
	 output [3:0] CMP_CLK_PHASE, // Comparator Clock Phase register value (0-15).
	 output [2:0] TMB_TX_MODE,    // TMB transmit mode (2-bits, 0: comparator data, 1: fixed patterns, 2: counters, 3: randoms).
	 output [29:0] LAY1_TO_6_HALF_STRIP, // TMB half strips for injecting patterns into the optical serial data stream
	 output [5:0] LAYER_MASK, // layer mask to indicate which layers to use for active half strips.
	 output reg JDAQ_RATE     //DAQ Rate selection: 0 = 1GbE (1.25Gbps line rate), 1 = 2.56GbE (3.2Gbps line rate).
	 );

   reg dshift;
   reg we,pre_we,pre_rd;
   reg we21,pre_we21,pre_rd22;
	
//
// BSCAN signals
//
   wire capture1;
   wire drck1;
   wire jreset1;
   wire runtest1;
   wire jsel1;
   wire jshift1;
   wire tck1;
   wire tdi1;
   wire tms1;
   wire update1;
   wire tdo1;
   wire capture2;
   wire drck2;
   wire jreset2;
   wire runtest2;
   wire jsel2;
   wire jshift2;
   wire tck2;
   wire tdi2;
   wire tms2;
   wire update2;
   wire tdo2;
	wire clrf;
	wire p_in;


	wire [63:0] f; //JTAG functions (one hot);
	wire f11;
	wire lxdlyout,prbout,dsy7,dmy2,dmy3,dmy4,dmy5,dmy6,dmy7,dmy8,dmy9,dmy10,dmy11,dmy12;
	wire tdof23,tdof5,tdof6,tdof8,tdof9,tdofa,tdofc,tdofe,tdof10,tdof11,tdof14,tdof15,tdof16,tdof17,tdof18,tdof1c,tdof1d,tdof1e,tdof1f,tdof24;
	wire [31:16] status_h;
   wire [6:1] bky_mask;
	wire [6:0] nsamp;
	reg [7:0] clr_pip;
	reg f18dly, f19dly;
	reg f32dly, f33dly;
	reg clr_cal_mode, set_cal_mode;
	reg set_rate_1_25, set_rate_3_2;
	reg [15:0] spi_rtn_reg;
	
	assign SAMP_MAX = nsamp-1;
	assign JC_ADC_CNFG = f[14];
	
	assign tdo2 = (tdof23 | tdof5 |  tdof6 | dsy7 | tdof8 | tdof9 | tdofa | tdofb | tdofc | tdofe | tdof10 | tdof11 | tdof14 | tdof15 | tdof16 | tdof17 | tdof18 | tdof1c | tdof1d | tdof1e | tdof1f | tdof24);
   assign {COMP_TIME,COMP_MODE} = status_h[20:16];
	assign status_h[31:21] = {5'b10110,XL1DLYSET,LOADPBLK};
	
	assign JTAG_SYS_RST  = f[1];  // System Reset JTAG command (like power on reset without reprogramming)
	assign ADC_INIT = f[13];      // ADC init JTAG command
	assign RSTRT_PIPE = f[15];    // Restart pipeline JTAG command
	assign BPI_RESET  = f[25];    // Reset BPI interface JTAG command
	assign BPI_DISABLE = f[26];   // Disable BPI processing
	assign BPI_ENABLE  = f[27];   // Enable BPI processing
	assign p_in = f[1] | f[13] | f[15] | f[18] | f[19] | f[25] | f[32] | f[33];  // JTAG_SYS_RST, ADC_Init, Restart pipeline, Cal mode, BPI_Reset and JDAQ_RATE JTAG commands are to be auto reset;
	assign clrf = clr_pip[4] & p_in; // auto reset functions last 5 clocks then clear
	
	assign JTAG_RD_MODE = f[6];    // JTAG readout mode when reading ADC data
//
//
// JTAG to SPI interface for Comparator DAC, Calibration DAC and MAX 1271 ADC
//
	assign CDAC_ENB = (dshift & jsel2 & f[4]);
	assign CALDAC_ENB = (dshift & jsel2 & f[34]);
	assign CALADC_ENB = (dshift & jsel2 & f[35]);
	assign SPI_CK = drck2 & (CDAC_ENB | CALDAC_ENB | CALADC_ENB);
	assign SPI_DAT = tdi2 & (CDAC_ENB | CALDAC_ENB | CALADC_ENB);
	
	
 
 /////////////////////////////////////////////////////////////////////////////
 //                                                                         //
 //  JTAG Access Ports for user function in the fabric (up to 4 interfaces) //
 //                                                                         //
 /////////////////////////////////////////////////////////////////////////////
 
 
   BSCAN_VIRTEX6 #(.DISABLE_JTAG("FALSE"),.JTAG_CHAIN(1))  // User 1 for instruction decodes
   BSCAN_user1 (
      .CAPTURE(capture1), // 1-bit output CAPTURE output from TAP controller
      .DRCK(drck1),       // 1-bit output Data register output for USER functions
      .RESET(jreset1),    // 1-bit output Reset output for TAP controller
      .RUNTEST(runtest1), // 1-bit output State output asserted when TAP controller is in Run Test Idle state.
      .SEL(jsel1),        // 1-bit output USER active output
      .SHIFT(jshift1),    // 1-bit output SHIFT output from TAP controller
      .TCK(tck1),         // 1-bit output Scan Clock output. Fabric connection to TAP Clock pin.
      .TDI(tdi1),         // 1-bit output TDI output from TAP controller
      .TMS(tms1),         // 1-bit output Test Mode Select input. Fabric connection to TAP.
      .UPDATE(update1),   // 1-bit output UPDATE output from TAP controller
      .TDO(tdo1)          // 1-bit input Data input for USER function
   );
  
   BSCAN_VIRTEX6 #(.DISABLE_JTAG("FALSE"),.JTAG_CHAIN(2))  // User 2 for data I/O
   BSCAN_user2 (
      .CAPTURE(capture2), // 1-bit output CAPTURE output from TAP controller
      .DRCK(drck2),       // 1-bit output Data register output for USER functions
      .RESET(jreset2),    // 1-bit output Reset output for TAP controller
      .RUNTEST(runtest2), // 1-bit output State output asserted when TAP controller is in Run Test Idle state.
      .SEL(jsel2),        // 1-bit output USER active output
      .SHIFT(jshift2),    // 1-bit output SHIFT output from TAP controller
      .TCK(tck2),         // 1-bit output Scan Clock output. Fabric connection to TAP Clock pin.
      .TDI(tdi2),         // 1-bit output TDI output from TAP controller
      .TMS(tms2),         // 1-bit output Test Mode Select input. Fabric connection to TAP.
      .UPDATE(update2),   // 1-bit output UPDATE output from TAP controller
      .TDO(tdo2)          // 1-bit input Data input for USER function
   );
	
	always @(posedge FSTCLK) dshift <= jshift2;  // Shift state delayed by 6.25ns
//
// JTAG Instruction decode Uses User 1 BSCAN signals
//
   instr_dcd
	instr_dcd1(
	   .TCK(tck1),         // TCK for update register
      .DRCK(drck1),       // Data Reg Clock
      .SEL(jsel1),        // User 1 mode active
      .TDI(tdi1),         // Serial Test Data In
      .UPDATE(update1),   // Update state
      .SHIFT(jshift1),    // Shift state
      .RST(RST),          // Reset default state
      .CLR(clrf),          // Clear the current function
      .F(f),              // Function decode output (one hot)
      .TDO(tdo1));        // Serial Test Data Out
		
		
//
// pipline to set length of auto reset signals
//
   always @(posedge CLK20) begin
	    clr_pip <= {clr_pip[6:0],p_in};
	end
  
//
// JTAG User Functions  Uses User 2 BSCAN signals
//
//
// JTAG Extra L1A Delay Register
//
   user_wr_reg #(.width(2), .def_value(2'b01))
   load_xl1dly(
	   .TCK(tck2),         // TCK for update register
      .DRCK(drck2),        // Data Reg Clock
      .FSEL(f[5]),        // Function select
      .SEL(jsel2),        // User 2 mode active
      .TDI(tdi2),          // Serial Test Data In
      .DSY_IN(tdi2),       // Serial Daisy chained data in
      .SHIFT(jshift2),      // Shift state
      .UPDATE(update2),    // Update state
      .RST(RST),          // Reset default state
      .DSY_CHAIN(f[7]),   // Daisy chain mode
      .PO(XL1DLYSET),     // Parallel output
      .TDO(tdof5),        // Serial Test Data Out
      .DSY_OUT(lxdlyout));// Daisy chained serial data out

//
// JTAG Pre block end register  (Not needed in DCFEB -- should remove but it is part of daisy chain for F7)
//
  user_wr_reg #(.width(4), .def_value(4'h9))
   load_preblk(
	   .TCK(tck2),         // TCK for update register
      .DRCK(drck2),        // Data Reg Clock
      .FSEL(f[8]),        // Function select
      .SEL(jsel2),        // User 2 mode active
      .TDI(tdi2),         // Serial Test Data In
      .DSY_IN(lxdlyout),  // Serial Daisy chained data in
      .SHIFT(jshift2),      // Shift state
      .UPDATE(update2),    // Update state
      .RST(RST),          // Reset default state
      .DSY_CHAIN(f[7]),   // Daisy chain mode
      .PO(LOADPBLK),      // Parallel output
      .TDO(tdof8),        // Serial Test Data Out
      .DSY_OUT(prbout));  // Daisy chained serial data out

//
// JTAG Comparator Mode and Timing bits Register
//
   user_wr_reg #(.width(5), .def_value(5'b00110))
   comparator(
	   .TCK(tck2),         // TCK for update register
      .DRCK(drck2),        // Data Reg Clock
      .FSEL(f[9]),        // Function select
      .SEL(jsel2),        // User 2 mode active
      .TDI(tdi2),         // Serial Test Data In
      .DSY_IN(prbout),    // Serial Daisy chained data in
      .SHIFT(jshift2),      // Shift state
      .UPDATE(update2),    // Update state
      .RST(RST),          // Reset default state
      .DSY_CHAIN(f[7]),   // Daisy chain mode
      .PO(status_h[20:16]), // Parallel output
      .TDO(tdof9),        // Serial Test Data Out
      .DSY_OUT(dsy7));    // Daisy chained serial data out
		
//
// Calibration mode
//
   always @(posedge CLK40) begin
		 f18dly <= f[18] & jsel2 & update2;              // only at update2
		 f19dly <= f[19] & jsel2 & update2;              // only at update2
		 clr_cal_mode <= ~(f[18] & jsel2 & update2) & f18dly;  
		 set_cal_mode <= ~(f[19] & jsel2 & update2) & f19dly;
		 if(clr_cal_mode || RST)
			CAL_MODE <= 1'b0;
		 else if(set_cal_mode)
			CAL_MODE <= 1'b1;
		 else
			CAL_MODE <= CAL_MODE;
	end
	
//
// JTAG Buckeye mask register for which amplifiers are in the shift loop.
//
	
   user_wr_reg #(.width(6), .def_value(6'b111111))
   buckeye_mask(
	   .TCK(tck2),         // TCK for update register
      .DRCK(drck2),        // Data Reg Clock
      .FSEL(f[10]),       // Function select
      .SEL(jsel2),        // User 2 mode active
      .TDI(tdi2),         // Serial Test Data In
      .DSY_IN(1'b0),      // Serial Daisy chained data in
      .SHIFT(jshift2),      // Shift state
      .UPDATE(update2),    // Update state
      .RST(RST),          // Reset default state
      .DSY_CHAIN(1'b0),   // Daisy chain mode
      .PO(bky_mask),      // Parallel output
      .TDO(tdofa),        // Serial Test Data Out
      .DSY_OUT(dmy2));    // Daisy chained serial data out

//
// JTAG Buckeye shift clocks data mutiplexers
//
   bky_shift 
	bky_shift1 (
		.DRCK(drck2),        // Data Reg Clock
		.SEL(jsel2),         // User 2 mode active
		.F(f[11]),          // Function select
		.TDI(tdi2),         // Serial Test Data In
		.MASK(bky_mask),    // Mask of which amplifiers to include in shift loop
		.SHIFT(dshift),     // Shift state
		.DRTN(BKY_RTN),     // Serial data returned from amplifiers
		.DSND(TO_BKY),      // Serial data sent to amplifiers
		.BCLK(BKY_CLK),     // Shift clock for amplifiers
		.TDO(tdofb)        // Test data out of the complete loop
		);
	
//
// Status capture and shift
//
   user_cap_reg #(.width(32))
   status1(
      .DRCK(drck2),        // Data Reg Clock
      .FSH(f[2]),         // Shift Function
      .FCAP(f[3]),        // Capture Function
      .SEL(jsel2),        // User 2 mode active
      .TDI(tdi2),          // Serial Test Data In
      .SHIFT(jshift2),      // Shift state
      .CAPTURE(capture2),  // Capture state
      .RST(RST),          // Reset default state
		.BUS({status_h,DCFEB_STATUS}), // Bus to capture
      .TDO(tdof23));      // Serial Test Data Out

//
// ADC Mask register
//
   user_wr_reg #(.width(12), .def_value(12'hFFF))
   adc_mask1(
	   .TCK(tck2),         // TCK for update register
      .DRCK(drck2),        // Data Reg Clock
      .FSEL(f[12]),       // Function select
      .SEL(jsel2),        // User 2 mode active
      .TDI(tdi2),          // Serial Test Data In
      .DSY_IN(1'b0),      // Serial Daisy chained data in
      .SHIFT(jshift2),      // Shift state
      .UPDATE(update2),    // Update state
      .RST(RST),          // Reset default state
      .DSY_CHAIN(1'b0),   // Daisy chain mode
      .PO(ADC_MASK),      // Parallel output
      .TDO(tdofc),        // Serial Test Data Out
      .DSY_OUT(dmy3));    // Daisy chained serial data out

//
// ADC Memory word register (data to write to ADC configuration memory)
//
   user_wr_reg #(.width(26), .def_value(26'h0000000))
   adc_mem1(
	   .TCK(tck2),         // TCK for update register
      .DRCK(drck2),        // Data Reg Clock
      .FSEL(f[14]),       // Function select
      .SEL(jsel2),        // User 2 mode active
      .TDI(tdi2),          // Serial Test Data In
      .DSY_IN(1'b0),      // Serial Daisy chained data in
      .SHIFT(jshift2),      // Shift state
      .UPDATE(update2),    // Update state
      .RST(RST),          // Reset default state
      .DSY_CHAIN(1'b0),   // Daisy chain mode
      .PO(ADC_MEM),       // Parallel output
      .TDO(tdofe),        // Serial Test Data Out
      .DSY_OUT(dmy4));    // Daisy chained serial data out
		
//
// Pipeline Depth register
//
   user_wr_reg #(.width(9), .def_value(9'd100))
   pipe_depth1(
	   .TCK(tck2),         // TCK for update register
      .DRCK(drck2),        // Data Reg Clock
      .FSEL(f[16]),       // Function select
      .SEL(jsel2),        // User 2 mode active
      .TDI(tdi2),          // Serial Test Data In
      .DSY_IN(1'b0),      // Serial Daisy chained data in
      .SHIFT(jshift2),      // Shift state
      .UPDATE(update2),    // Update state
      .RST(RST),          // Reset default state
      .DSY_CHAIN(1'b0),   // Daisy chain mode
      .PO(PDEPTH),        // Parallel output
      .TDO(tdof10),        // Serial Test Data Out
      .DSY_OUT(dmy5));    // Daisy chained serial data out
		
		
//
// TTC source register
//
//  FF_EMU_mode    = 2'b00, 
//  FF_EEM_mode    = 2'b01, 
//  Skew_Clr_mode  = 2'b10;

   user_wr_reg #(.width(2), .def_value(2'b10)) // default is Skewclear mode
   ttc_src1(
	   .TCK(tck2),         // TCK for update register
      .DRCK(drck2),        // Data Reg Clock
      .FSEL(f[17]),       // Function select
      .SEL(jsel2),        // User 2 mode active
      .TDI(tdi2),          // Serial Test Data In
      .DSY_IN(1'b0),      // Serial Daisy chained data in
      .SHIFT(jshift2),      // Shift state
      .UPDATE(update2),    // Update state
      .RST(RST),          // Reset default state
      .DSY_CHAIN(1'b0),   // Daisy chain mode
      .PO(TTC_SRC),        // Parallel output
      .TDO(tdof11),        // Serial Test Data Out
      .DSY_OUT(dmy6));    // Daisy chained serial data out
		
		
		
//
// ADC Write enable for configuration memory
//
   always @(posedge CLK20) begin
		 pre_we <= f[14] & jsel2 & update2;              // only at update2
		 we     <= ~(f[14] & jsel2 & update2) & pre_we;  // generate trailing edge pulse one clock long
		 ADC_WE <= we;                                 // delay write enable one clock cycle
	end

//
// ADC Data readback  capture and shift
//
   user_cap_reg #(.width(192))
   ADCread1(
      .DRCK(drck2),        // Data Reg Clock
      .FSH(1'b0),         // Shift Function
      .FCAP(f[6]),        // Capture Function
      .SEL(jsel2),        // User 2 mode active
      .TDI(tdi2),          // Serial Test Data In
      .SHIFT(jshift2),      // Shift state
      .CAPTURE(capture2),  // Capture state
      .RST(RST),          // Reset default state
		.BUS(ADCDATA), // Bus to capture
      .TDO(tdof6));      // Serial Test Data Out
	
//
// Read enable for ADC data FIFO -- advance on each JTAG read.
//
   always @(posedge CLK120) begin
		 pre_rd <= f[6] & jsel2 & update2;              // only at update2
		 RDFIFO <= (f[6] & jsel2 & update2) & ~pre_rd;  // generate leading edge pulse one clock long
	end
	
//
// Number of samples to read register
//
   user_wr_reg #(.width(7), .def_value(7'd8))
   nsamples1(
	   .TCK(tck2),         // TCK for update register
      .DRCK(drck2),        // Data Reg Clock
      .FSEL(f[20]),       // Function select
      .SEL(jsel2),        // User 2 mode active
      .TDI(tdi2),          // Serial Test Data In
      .DSY_IN(1'b0),      // Serial Daisy chained data in
      .SHIFT(jshift2),      // Shift state
      .UPDATE(update2),    // Update state
      .RST(RST),          // Reset default state
      .DSY_CHAIN(1'b0),   // Daisy chain mode
      .PO(nsamp),        // Parallel output
      .TDO(tdof14),        // Serial Test Data Out
      .DSY_OUT(dmy7));    // Daisy chained serial data out

//
// BPI interface FIFO word (16 bit word to be writting into the FIFO)
// The write enable for the FIFO is generated automatically
//
   user_wr_reg #(.width(16), .def_value(16'h0000))
   BPI_wrt_FIFO_Jreg(
	   .TCK(tck2),         // TCK for update register
      .DRCK(drck2),        // Data Reg Clock
      .FSEL(f[21]),       // Function select
      .SEL(jsel2),        // User 2 mode active
      .TDI(tdi2),          // Serial Test Data In
      .DSY_IN(1'b0),      // Serial Daisy chained data in
      .SHIFT(jshift2),      // Shift state
      .UPDATE(update2),    // Update state
      .RST(RST),          // Reset default state
      .DSY_CHAIN(1'b0),   // Daisy chain mode
      .PO(BPI_WRT_FIFO),       // Parallel output
      .TDO(tdof15),        // Serial Test Data Out
      .DSY_OUT(dmy8));    // Daisy chained serial data out
		
//
// BPI  Write enable for BPI write FIFO
//
   always @(posedge CLK40) begin
		 pre_we21 <= f[21] & jsel2 & update2;              // only at update2
		 we21     <= ~(f[21] & jsel2 & update2) & pre_we21;  // generate tailing edge pulse one clock long
		 BPI_WE <= we21;                                 // delay write enable one clock cycle
	end

//
// BPI Data readback  capture and shift from BPI_RBK_FIFO
//
   user_cap_reg #(.width(16))
   BPI_rbk_FIFO_Jreg(
      .DRCK(drck2),        // Data Reg Clock
      .FSH(1'b0),         // Shift Function
      .FCAP(f[22]),        // Capture Function
      .SEL(jsel2),        // User 2 mode active
      .TDI(tdi2),          // Serial Test Data In
      .SHIFT(jshift2),      // Shift state
      .CAPTURE(capture2),  // Capture state
      .RST(RST),          // Reset default state
		.BUS(BPI_RBK_FIFO), // Bus to capture
      .TDO(tdof16));      // Serial Test Data Out
	
//
// Read enable for BPI data FIFO -- advance on each JTAG read.
//
   always @(posedge CLK40) begin
		 pre_rd22 <= f[22] & jsel2 & update2;              // only at update2
		 BPI_RDENA <= (f[22] & jsel2 & update2) & ~pre_rd22;  // generate leading edge pulse one clock long
	end
		
//
// BPI status register capture and shift
//
   user_cap_reg #(.width(16))
   BPI_status_Jreg(
      .DRCK(drck2),        // Data Reg Clock
      .FSH(1'b0),         // Shift Function
      .FCAP(f[23]),        // Capture Function
      .SEL(jsel2),        // User 2 mode active
      .TDI(tdi2),          // Serial Test Data In
      .SHIFT(jshift2),      // Shift state
      .CAPTURE(capture2),  // Capture state
      .RST(RST),          // Reset default state
		.BUS(BPI_STATUS), // Bus to capture
      .TDO(tdof17));      // Serial Test Data Out
		
		
//
// BPI timer register capture and shift
//
   user_cap_reg #(.width(32))
   BPI_timer_Jreg(
      .DRCK(drck2),        // Data Reg Clock
      .FSH(1'b0),         // Shift Function
      .FCAP(f[24]),        // Capture Function
      .SEL(jsel2),        // User 2 mode active
      .TDI(tdi2),          // Serial Test Data In
      .SHIFT(jshift2),      // Shift state
      .CAPTURE(capture2),  // Capture state
      .RST(RST),          // Reset default state
		.BUS(BPI_TIMER), // Bus to capture
      .TDO(tdof18));      // Serial Test Data Out

//
// Comparator Clock phase register
//

   user_wr_reg #(.width(4), .def_value(4'h0)) // default is no phase shift
   cmp_clock_phase(
	   .TCK(tck2),         // TCK for update register
      .DRCK(drck2),        // Data Reg Clock
      .FSEL(f[28]),       // Function select
      .SEL(jsel2),        // User 2 mode active
      .TDI(tdi2),          // Serial Test Data In
      .DSY_IN(1'b0),      // Serial Daisy chained data in
      .SHIFT(jshift2),      // Shift state
      .UPDATE(update2),    // Update state
      .RST(RST),          // Reset default state
      .DSY_CHAIN(1'b0),   // Daisy chain mode
      .PO(CMP_CLK_PHASE),        // Parallel output
      .TDO(tdof1c),        // Serial Test Data Out
      .DSY_OUT(dmy9));    // Daisy chained serial data out
		

//
// TMB optical path transmit mode register
// mode: function
// -----------------
// 0: comapator data
// 1: fixed patterns
// 2: counters
// 3: randoms
// 4: comapator data
// 5: half strips
//

   user_wr_reg #(.width(3), .def_value(3'b000)) // default is comparator data
   tmb_tx_mode(
	   .TCK(tck2),         // TCK for update register
      .DRCK(drck2),        // Data Reg Clock
      .FSEL(f[29]),       // Function select
      .SEL(jsel2),        // User 2 mode active
      .TDI(tdi2),          // Serial Test Data In
      .DSY_IN(1'b0),      // Serial Daisy chained data in
      .SHIFT(jshift2),      // Shift state
      .UPDATE(update2),    // Update state
      .RST(RST),          // Reset default state
      .DSY_CHAIN(1'b0),   // Daisy chain mode
      .PO(TMB_TX_MODE),    // Parallel output
      .TDO(tdof1d),        // Serial Test Data Out
      .DSY_OUT(dmy10));    // Daisy chained serial data out
		
//
// TMB half strip settings for injecting patterns into serial data stream
// 

   user_wr_reg #(.width(30), .def_value(30'h00000000)) // default is no half strips
   tmb_half_strip(
	   .TCK(tck2),         // TCK for update register
      .DRCK(drck2),        // Data Reg Clock
      .FSEL(f[30]),       // Function select
      .SEL(jsel2),        // User 2 mode active
      .TDI(tdi2),          // Serial Test Data In
      .DSY_IN(1'b0),      // Serial Daisy chained data in
      .SHIFT(jshift2),      // Shift state
      .UPDATE(update2),    // Update state
      .RST(RST),          // Reset default state
      .DSY_CHAIN(1'b0),   // Daisy chain mode
      .PO(LAY1_TO_6_HALF_STRIP),    // Parallel output
      .TDO(tdof1e),        // Serial Test Data Out
      .DSY_OUT(dmy11));    // Daisy chained serial data out
		
		
//
// TMB layer mask for indicating active layers for injecting patterns into serial data stream
// 

   user_wr_reg #(.width(6), .def_value(6'b111111)) // default is all layers active
   tmb_layer_mask(
	   .TCK(tck2),         // TCK for update register
      .DRCK(drck2),        // Data Reg Clock
      .FSEL(f[31]),       // Function select
      .SEL(jsel2),        // User 2 mode active
      .TDI(tdi2),          // Serial Test Data In
      .DSY_IN(1'b0),      // Serial Daisy chained data in
      .SHIFT(jshift2),      // Shift state
      .UPDATE(update2),    // Update state
      .RST(RST),          // Reset default state
      .DSY_CHAIN(1'b0),   // Daisy chain mode
      .PO(LAYER_MASK),    // Parallel output
      .TDO(tdof1f),        // Serial Test Data Out
      .DSY_OUT(dmy12));    // Daisy chained serial data out
		
//
// DAQ rate selection
//
   always @(posedge CLK40) begin
		 f32dly <= f[32] & jsel2 & update2;              // only at update2
		 f33dly <= f[33] & jsel2 & update2;              // only at update2
		 set_rate_1_25 <= ~(f[32] & jsel2 & update2) & f32dly;  // trailing edge of update2
		 set_rate_3_2  <= ~(f[33] & jsel2 & update2) & f33dly;  // trailing edge of update2
		 if(set_rate_1_25)
			JDAQ_RATE <= 1'b0;
		 else if(set_rate_3_2 || RST)
			JDAQ_RATE <= 1'b1;
		 else
			JDAQ_RATE <= JDAQ_RATE;
	end
	
//
// SPI Return data register
//
  
  always @(posedge drck2 or posedge RST) begin // intermediate shift register
    if(RST)
	   spi_rtn_reg <= 16'h0000;           // default
    else
	   if(dshift & jsel2 & f[35])
	     spi_rtn_reg <= {spi_rtn_reg[14:0],SPI_RTN}; // Shift left
		else
		  spi_rtn_reg <= spi_rtn_reg;                  // Hold
  end
//
// SPI Return data register capture and shift
//
   user_cap_reg #(.width(16))
   SPI_Rtn_Reg(
      .DRCK(drck2),        // Data Reg Clock
      .FSH(1'b0),         // Shift Function
      .FCAP(f[36]),        // Capture Function
      .SEL(jsel2),        // User 2 mode active
      .TDI(tdi2),          // Serial Test Data In
      .SHIFT(jshift2),      // Shift state
      .CAPTURE(capture2),  // Capture state
      .RST(RST),          // Reset default state
		.BUS(spi_rtn_reg), // Bus to capture
      .TDO(tdof24));      // Serial Test Data Out

	

endmodule
